function [MA_]=first_order(M_,MA_)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% first_order.m
%
% This file produces the coefficients for a first-order approximation 
% adapted from dynAIMsolver1.m of dynare version 4.2.1
% 
% solution form: b(i) = ALPHA * b(i-1) + BETA_1ST * U(i), where U(i) = 0^i
%
% dynAIMsovler1 is itself not called because we
% need some intermediate results for later computation, i.e. theAIM_H
% matrix, and we don't need the ordered gy, what we need is the unordered
% gy, MA_.ALPHA.
% 
%Included in the package nonlinear_MA
%
%THIS VERSION: 1.0.1 October 12, 201
%
%Copyright: Hong Lan and Alexander Meyer-Gohde
%
%You are free to use/modify/redistribute this program so long as original
%authorship credit is given and you in no way impinge on its free
%distribution
%This software is provided as is with no guarantees of any kind
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% no. of equations                                           
neq  = size(MA_.df1,1);
% no. of lags
lags = M_.maximum_endo_lag;
% no. of leads
leads= M_.maximum_endo_lead;


  % AIM H matrix
  MA_.theAIM_H=zeros(neq,MA_.klen*neq);%%% AMG 4/20: declare theAIM_H first. Ensures that it has the right dimension.
   % MA_.theAIM_H(:,iyr0) = MA_.df1(:,nonzeros(iyv)); %%% AMG 4/20: Made use of iyr0 from above
   MA_.theAIM_H(:,MA_.iyv>0) = MA_.df1(:,MA_.iyv(MA_.iyv>0));
  % SPAmalg uses this in zero tests
    MA_.condn  = 1.e-10;
  % allow unit roots
    MA_.uprbnd = 1 + 1.e-6;
  % AIM
    [MA_.bb,MA_.rts,MA_.ia,MA_.nexact,MA_.nnumeric,MA_.lgroots,MA_.aimcode] =...
          SPAmalg(MA_.theAIM_H, neq,...
          lags, leads, MA_.condn, MA_.uprbnd);
    % homogeneous solution
      MA_.ALPHA = MA_.bb;
      MA_.ALPHA(abs(MA_.ALPHA)<MA_.condn)=0;
    % inhomogeneous solution
      if M_.exo_nbr % if there are exogenous shocks then derive gu for the shocks:
         % get H0 and H+1=HM
         MA_.theH0 = MA_.theAIM_H (:,lags*neq+1: (lags+1)*neq);
         MA_.theHP = MA_.theAIM_H (:,(lags+1)*neq+1: (lags+2)*neq);
         theAIM_Psi = - MA_.df1(:, size(MA_.iyv(MA_.iyv>0))+1:end);% 
         %? = inv(H0 + H1B1)
         MA_.theAIM_phi = (MA_.theH0 + MA_.theHP*sparse(MA_.bb(:,(lags-1)*neq+1:end)))\eye( neq);
         MA_.AIM_ghu = MA_.theAIM_phi * theAIM_Psi;
      else
         MA_.AIM_ghu = [];
      end
      MA_.BETA_1 = MA_.AIM_ghu;
    MA_.BETA_1(abs(MA_.BETA_1)<MA_.condn)=0;
     %For sylvesters
     MA_.syl_lhs_jacobianlag    = MA_.theAIM_H(:,1:M_.endo_nbr);
     MA_.syl_lhs_jacobian0      = MA_.theAIM_H(:,M_.endo_nbr+1:2*M_.endo_nbr);
     MA_.syl_lhs_jacobianlead   = MA_.theAIM_H(:,2*M_.endo_nbr+1:end);
   % build up lhs coefficients
     MA_.syl_AA = MA_.syl_lhs_jacobian0 + MA_.syl_lhs_jacobianlead*MA_.ALPHA;
     MA_.syl_BB = MA_.syl_lhs_jacobianlead;

%This builds the term X_{\tilde{epsilon}} using the first-order
%solution. It measure the dependence of the systems' variables to a shock
%in t+1. Only y_{t+1} has such a dependence.
MA_.Y_eps_1=[zeros(2*M_.endo_nbr,M_.exo_nbr);MA_.BETA_1;zeros(M_.exo_nbr,M_.exo_nbr)];     
%This evaluates the matrix polynomial associated with the homgenous part of
%the recursion at one. This must be of full-rank as unit-roots are ruled
%out by assumption
MA_.matrix_polynomial_one=MA_.theAIM_H(:,1:M_.endo_nbr)+MA_.theAIM_H(:,M_.endo_nbr+1:2*M_.endo_nbr)+MA_.theAIM_H(:,2*M_.endo_nbr+1:3*M_.endo_nbr);   
   

end

