function [MA_]=second_order(M_,MA_)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% second_order.m
%
% This file produces the coefficients for a second-order approximation in 
% addition to those produced by first_order.m
% 
%Included in the package nonlinear_MA
%
%THIS VERSION: 1.0.1 October 12, 201
%
%Copyright: Hong Lan and Alexander Meyer-Gohde
%
%You are free to use/modify/redistribute this program so long as original
%authorship credit is given and you in no way impinge on its free
%distribution
%This software is provided as is with no guarantees of any kind
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% build up shifting matrix gamma(1), where (1) indicates that this shifting
% matrix bridges first order solution to the second order. 
% build up gamma(1), such that
%[y_(i-1)',y_(i)',y_(i+1)',U_(i)']'=gamma(1)*[y_(i-1)', U_(i)']'. 
MA_.GAMMA_1 = [ eye(M_.endo_nbr)               zeros(M_.endo_nbr,M_.exo_nbr)
                                MA_.ALPHA                         MA_.BETA_1
                                MA_.ALPHA^2                     MA_.ALPHA*MA_.BETA_1
                       zeros(M_.exo_nbr,M_.endo_nbr)   eye(M_.exo_nbr)               ];

% build up shifting matrix delta(1), such that
% [y_(i)',U_(i+1)']'=delta(1)*[y_(i-1)', U_(i)']'. 
MA_.DELTA_1 = [ MA_.ALPHA                       MA_.BETA_1
                       zeros(M_.exo_nbr,M_.endo_nbr)   zeros(M_.exo_nbr)  ];

% build the remaining two order-dependent matrices in the Sylvester
syl_DD = -MA_.df2*kron(MA_.g_picker*MA_.GAMMA_1,MA_.g_picker*MA_.GAMMA_1);

%syl_CC = kron(MA_.DELTA_1,MA_.DELTA_1);
% Call Dynare's Sylvester caller
[waste, MA_.BETA_2,MA_.par] = gensylv(2,MA_.syl_AA,MA_.syl_BB,MA_.DELTA_1,syl_DD);%can take advantage of Kronecker input on C in Dynare's Sylvester solver
%MA_.BETA_2=sylvester3(MA_.syl_AA,MA_.syl_BB,syl_CC,syl_DD);
MA_.BETA_2(abs(MA_.BETA_2)<MA_.condn)=0;

%Can check Dynare's sylvester solver using the Kronecker representation
%MA_.BETA_2_alt=reshape((kron(eye(size(syl_CC)),MA_.syl_AA)+kron(syl_CC',MA_.syl_BB))\vec(syl_DD),[M_.endo_nbr,(M_.endo_nbr+M_.exo_nbr)^2]);

%This builds the term Y_{\tilde{epsilon}^2} using the second-order
%solution. It measure the second-order dependence of the systems' variables to a shock
%in t+1. Only y_{t+1} has such a dependence.
MA_.Y_eps_2=[zeros(2*M_.endo_nbr,M_.exo_nbr^2);MA_.BETA_2*kron([zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr,M_.exo_nbr)],[zeros(M_.endo_nbr,M_.exo_nbr);eye(M_.exo_nbr,M_.exo_nbr)]);zeros(M_.exo_nbr,M_.exo_nbr^2)];  

%Now calculate Y_sigma_sigma
MA_.Y_sigma_2=  -(2*MA_.matrix_polynomial_one)\((MA_.df2*kron(MA_.g_picker*MA_.Y_eps_1,MA_.g_picker*MA_.Y_eps_1)+MA_.df1*MA_.g_picker*MA_.Y_eps_2)*vec(M_.Sigma_e));
MA_.Y_sigma_2(abs(MA_.Y_sigma_2)<MA_.condn)=0;